#include <propeller.h>
#include "ws2812.h"

//#define RING
//#define STRIP
#define LEDS

class ws2812_Animation {
public:
    virtual void update();
};

class ws2812_FillWheel : public ws2812_Animation {
public:
    ws2812_FillWheel(uint32_t *colors, int count, int samples)
      : m_colors(colors), m_count(count), m_samples(samples), m_i(0), m_di(1), m_j(0) {}
    ~ws2812_FillWheel() {}
    void update();
private:
    uint32_t *m_colors;
    int m_count;
    int m_samples;
    int m_i;
    int m_di;
    int m_j;
};

void ws2812_FillWheel::update()
{
    m_colors[m_i] = ws2812x::Wheel(m_j * (255 / m_samples));
    m_i += m_di;
    if (m_i >= m_count) {
        m_di = -1;
        --m_i;
    }
    else if (m_i < 0) {
        m_di = 1;
        ++m_i;
    } 
    if (++m_j >= m_samples)
        m_j = 0;    
}

class ws2812_FillWheelDim : public ws2812_Animation {
public:
    ws2812_FillWheelDim(uint32_t *colors, int count, int samples, int brightness)
      : m_colors(colors), m_count(count), m_samples(samples), m_brightness(brightness), m_i(0), m_di(1), m_j(0) {}
    ~ws2812_FillWheelDim() {}
    void update();
private:
    uint32_t *m_colors;
    int m_count;
    int m_samples;
    int m_brightness;
    int m_i;
    int m_di;
    int m_j;
};

void ws2812_FillWheelDim::update()
{
    m_colors[m_i] = ws2812x::WheelDim(m_j * (255 / m_samples), m_brightness);
    m_i += m_di;
    if (m_i >= m_count) {
        m_di = -1;
        --m_i;
    }
    else if (m_i < 0) {
        m_di = 1;
        ++m_i;
    } 
    if (++m_j >= m_samples)
        m_j = 0;    
}

class ws2812_Chase : public ws2812_Animation {
public:
    ws2812_Chase(uint32_t *colors, int count, uint32_t *pattern, int pcount)
      : m_colors(colors), m_count(count), m_pattern(pattern), m_pcount(pcount), m_base(0) {}
    ~ws2812_Chase() {}
    void update();
private:
    uint32_t *m_colors;
    int m_count;
    uint32_t *m_pattern;
    int m_pcount;
    int m_base;
};

void ws2812_Chase::update()
{
    int idx = m_base;                       // start at base
    for (int i = 0; i < m_count; ++i) {     // loop through connected leds
        m_colors[i] = m_pattern[idx];       // update channel color
        if (++idx >= m_pcount)              // past end of list?
            idx = 0;                        // yes, reset
    }
    if (++m_base >= m_pcount)               // update the base for the next time
        m_base = 0;
}

#include <propeller.h>
#include "ws2812.h"

/* ring chain */
#define RING_PIN    15
#define RING_COUNT  16

/* strip chain */
#define STRIP_PIN   14
#define STRIP_COUNT 8

/* individual chain */
#define INDIV_PIN   13
#define INDIV_COUNT 4

#if 0

uint32_t Chakras[] = {
    COLOR_RED,
    COLOR_ORANGE,
    COLOR_YELLOW, 
    COLOR_GREEN,
    COLOR_BLUE,
    COLOR_INDIGO
};
const int Chackras_count = sizeof(Chakras) / sizeof(uint32_t);

int main(void)
{
    ws2812b driver;
    
    if (!driver.running())
        return 1;

#ifdef RING
    // 16 LED ring
    uint32_t ringColors[RING_COUNT];
    ws2812_FillWheelDim wheel(ringColors, RING_COUNT, RING_COUNT + 5, 128);
    ws2812_Chain ring(RING_PIN, ringColors, RING_COUNT);
#endif

#ifdef STRIP
    // 8 LED strip
    uint32_t stripColors[STRIP_COUNT];
    ws2812_Chase chase(stripColors, STRIP_COUNT, Chakras, Chackras_count);
    ws2812_Chain strip(STRIP_PIN, stripColors, STRIP_COUNT);
#endif
    
#ifdef LEDS
    // 4 individual LEDs
    uint32_t indivColors[INDIV_COUNT];
    ws2812_Chase chase2(indivColors, INDIV_COUNT, Chakras, Chackras_count);
    ws2812_Chain indiv(INDIV_PIN, indivColors, INDIV_COUNT);
#endif
    
    // animation array
    ws2812_Animation *animations[] = {
#ifdef RING
        &wheel,
#endif
#ifdef STRIP
        &chase,
#endif
#ifdef LEDS
        &chase2
#endif
    };

    // LED chain array
    ws2812_Chain *chains[] = {
#ifdef RING
        &ring,
#endif
#ifdef STRIP
        &strip,
#endif
#ifdef LEDS
        &indiv
#endif
    };

    for (;;) {
    
        // update each of the animations
        for (unsigned int i = 0; i < sizeof(animations) / sizeof(animations[0]); ++i)
            animations[i]->update();
            
        // update each of the LED chains
        for (unsigned int i = 0; i < sizeof(chains) / sizeof(chains[0]); ++i)
            driver.refresh(*chains[i]);
            
        // delay between frames
        ws2812b::pause(100);
    }
           
    return 0;
}
#endif

/* led chain */
#define LEDS_PIN    13
#define LEDS_COUNT  4

uint32_t Chakras[] = {
    COLOR_RED,
    COLOR_ORANGE,
    COLOR_YELLOW, 
    COLOR_GREEN,
    COLOR_BLUE,
    COLOR_INDIGO
};
const int Chackras_count = sizeof(Chakras) / sizeof(uint32_t);

int main(void)
{
    ws2812b driver;
    
    if (!driver.running())
        return 1;

    // 4 individual LEDs
    uint32_t ledsColors[LEDS_PIN];
    ws2812_Chase chase(ledsColors, LEDS_COUNT, Chakras, Chackras_count);
    ws2812_Chain leds(LEDS_PIN, ledsColors, LEDS_COUNT);
    
    for (;;) {
    
        // update the animation
        chase.update();
            
        // update the LED chain
        driver.refresh(leds);
            
        // delay between frames
        ws2812b::pause(100);
    }
           
    return 0;
}
