#include <propeller.h>
#include "ws2812.h"

// driver header structure
typedef struct {
    uint32_t    jmp_inst;
    uint32_t    resettix;
    uint32_t    bit0hi;
    uint32_t    bit0lo;
    uint32_t    bit1hi;
    uint32_t    bit1lo;
    uint32_t    swaprg;
} ws2812_hdr;

int ws2812x::m_ticks_per_ms = 0;

ws2812x::ws2812x(int usreset, int ns0h, int ns0l, int ns1h, int ns1l, int type)
 : m_command(0), m_cog(-1)
{
    extern uint32_t binary_ws2812_driver_dat_start[];
    ws2812_hdr *hdr = (ws2812_hdr *)binary_ws2812_driver_dat_start;
    uint32_t ustix;

    ustix = CLKFREQ / 1000000;          // ticks in 1us

    hdr->resettix = ustix * usreset;
    hdr->bit0hi   = ustix * ns0h / 1000;
    hdr->bit0lo   = ustix * ns0l / 1000;
    hdr->bit1hi   = ustix * ns1h / 1000;
    hdr->bit1lo   = ustix * ns1l / 1000;
    hdr->swaprg   = (type == kGRB);
    
    m_cog = cognew(hdr, &m_command);
}

void ws2812x::refresh(ws2812_Chain &chain)
{
    uint32_t cmd;
    cmd =  chain.pin()
        | (chain.count() << 8)
        | ((uint32_t)chain.colors() << 16);
    while (m_command)
        ;
    m_command = cmd;
}

ws2812x::~ws2812x()
{
    if (m_cog != -1)
        cogstop(m_cog);
}

uint32_t ws2812x::Wheel(int pos)
{
    uint32_t color;

// Creates color from 0 to 255 position input
// -- colors transition r-g-b back to r

    // red range
    if (pos < 85)
        color = COLOR(255-pos*3, pos*3, 0);
    
    // green range
    else if (pos < 170) {
        pos -= 85;
        color = COLOR(0, 255-pos*3, pos*3);
    }
    
    // blue range
    else {
        pos -= 170;
        color = COLOR(pos*3, 0, 255-pos*3);
    }
    
    return color;
}

uint32_t ws2812x::WheelDim(int pos, int brightness)
{
    uint32_t color;

// Creates color from 0 to 255 position input
// -- colors transition r-g-b back to r

    // red range
    if (pos < 85)
        color = COLORX(255-pos*3, pos*3, 0, brightness);
    
    // green range
    else if (pos < 170) {
        pos -= 85;
        color = COLORX(0, 255-pos*3, pos*3, brightness);
    }
    
    // blue range
    else {
        pos -= 170;
        color = COLORX(pos*3, 0, 255-pos*3, brightness);
    }
    
    return color;
}

void ws2812x::pause(int ms)
{
    if (!m_ticks_per_ms)
        m_ticks_per_ms = CLKFREQ / 1000;
    waitcnt(CNT + ms * m_ticks_per_ms);
}