#ifndef __WS2812_H__
#define __WS2812_H__

#include <stdint.h>

#define COLOR(r, g, b)      (((r) << 16) | ((g) << 8) | (b))
#define SCALE(x, l)         ((x) * (l) / 255)
#define COLORX(r, g, b, l)  ((SCALE(r, l) << 16) | (SCALE(g, l) << 8) | SCALE(b, l))

//                         RRGGBB
#define COLOR_BLACK      0x000000
#define COLOR_RED        0xFF0000
#define COLOR_GREEN      0x00FF00
#define COLOR_BLUE       0x0000FF
#define COLOR_WHITE      0xFFFFFF
#define COLOR_CYAN       0x00FFFF
#define COLOR_MAGENTA    0xFF00FF
#define COLOR_YELLOW     0xFFFF00
#define COLOR_CHARTREUSE 0x7FFF00
#define COLOR_ORANGE     0xFF6000
#define COLOR_AQUAMARINE 0x7FFFD4
#define COLOR_PINK       0xFF5F5F
#define COLOR_TURQUOISE  0x3FE0C0
#define COLOR_REALWHITE  0xC8FFFF
#define COLOR_INDIGO     0x3F007F
#define COLOR_VIOLET     0xBF7FBF
#define COLOR_MAROON     0x320010
#define COLOR_BROWN      0x0E0600
#define COLOR_CRIMSON    0xDC283C
#define COLOR_PURPLE     0x8C00FF

class ws2812_Chain {
public:
    ws2812_Chain(int pin, uint32_t *colors, int count)
      : m_pin(pin), m_colors(colors), m_count(count) {}
    ~ws2812_Chain() {}
    int pin() { return m_pin; }
    uint32_t *colors() { return m_colors; }
    int count() { return m_count; }
private:
    int m_pin;
    uint32_t *m_colors;
    int m_count;
};

class ws2812x {
public:
    enum {
        kRGB,
        kGRB    // for WS2812 and WS2812B
    };
    ws2812x(int usreset, int ns0h, int ns0l, int ns1h, int ns1l, int type);
    ~ws2812x();
    bool running() { return m_cog != -1; }
    void refresh(ws2812_Chain &chain);
    static uint32_t Wheel(int pos);
    static uint32_t WheelDim(int pos, int brightness);
    static void pause(int ms);
private:
    static int m_ticks_per_ms;
    volatile uint32_t m_command;
    int m_cog;
};

class ws2812 : public ws2812x {
public:
    ws2812() : ws2812x(50, 350, 800, 700, 600, kGRB) {}
    ~ws2812() {}
};

class ws2812b : public ws2812x {
public:
    ws2812b() : ws2812x(50, 350, 900, 900, 350, kGRB) {}
    ~ws2812b() {}
};

#endif
